
/** \file def.h
   - Project:     SOFTSUSY
   - Author:      Ben Allanach 
   - Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   - Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   - Description: Contains switches and default parameters

   $Log: def.h,v $
   Revision 1.4  2006/01/17 16:40:17  allanach
   Updated to 2.0.4: Higgs mass calculation corrected (problem in the charged
   Higgs contributions)

   Revision 1.3  2005/11/09 14:12:24  allanach
   Updated for SOFTSUSY 2.0.1 - cleaned everything up etc

   Revision 1.3  2005/07/26 10:51:22  allanach
   new method nmin added to DoubleVector and DoubleMatrix by Bernhardt

   Revision 1.2  2004/12/23 15:29:20  allanach
   Promoted INCLUDE_2_LOOP_SCALAR_CORRECTIONS to a global variable (in
   preparation for its control in the SUSY Les Houches Accord)

   Revision 1.36  2004/07/30 13:56:07  allanach
   Updated to PDG2004 central values

   Revision 1.35  2004/03/21 20:43:04  allanach
   Added alternative electroweak symmetry breaking conditions. Added
   possibility of EWSB=SUSY breaking boundary condition scale in SUSY Les
   Houches Accord. 

   Revision 1.34  2004/01/15 13:54:54  allanach
   New heaer style implemented

   Revision 1.33  2003/08/07 12:27:49  allanach
   Version relevant for producing numbers for Pietro

   Revision 1.32  2003/08/04 16:43:20  allanach
   GMU now made an input parameter to conform with SUSY Les Houches accord.
   root2 defined for efficiency.

   Revision 1.31  2003/07/24 15:30:42  allanach
   More changes to Les Houches interfacing: LH accord output only happens if LH
   accord stuff is input

   Revision 1.30  2003/07/24 14:55:28  allanach
   Implemented les Houches input and output properly in the usual command-line
   interface

   Revision 1.29  2003/07/22 09:16:04  allanach
   Added running MW, MZ to definition of drbar parameters

   Revision 1.28  2003/07/21 14:00:17  allanach
   MZ fully implemented as an input now. Kept MZ as the central PDG 2002 value,
   for defaults etc

   Revision 1.27  2003/07/21 13:10:16  allanach
   Removed MZ as constant and made it an input

   Revision 1.26  2003/07/21 08:47:28  allanach
   MW now part of MssmSoftsusy class, and prediction is implemented fully

   Revision 1.25  2003/07/18 14:39:20  allanach
   Implemented MW as a global variable (in preparation for predicting it),
   and also speed corrections in getVev and rhohat: allowing input of
   self-energies to remove their calculation several times

   Revision 1.24  2003/07/18 12:57:04  allanach
   Added comments on new meaning of SPARTICLE LOOP parameter

   Revision 1.23  2003/06/25 17:58:40  allanach
   Put in 2-loop higgs mass corrections (in third family approximation) as a
   default in softpars

   Revision 1.21  2003/06/05 09:17:19  allanach
   Started coding Les Houches Discord

   Revision 1.20  2003/05/22 12:45:41  allanach
   Fortran version of 2-loop Higgs corrections included with corrections:
   alpha_s (alpha_t + alpha_b) + alpha_t^2 + alpha_b^2 + alpha_t alpha_b

   Revision 1.19  2003/05/20 15:17:20  allanach
   With doxygen comments

   Revision 1.18  2003/03/28 15:57:15  allanach
   New parameters decide number of loops in Higgs mass and EWSB condition

   Revision 1.17  2003/03/07 11:30:34  allanach
   Removed fortran: converted to C++

   Revision 1.16  2003/02/21 13:02:07  allanach
   Changed headings to new conventions

   Revision 1.15  2003/02/18 15:53:10  allanach
   Removed redunant information

   Revision 1.14  2002/12/10 17:39:11  allanach
   Added 2-loop higgs calls

   Revision 1.12  2002/09/19 17:02:43  allanach
   Corrected alpha_s(MZ) value from PDG

   Revision 1.11  2002/09/19 16:22:03  allanach
   Latest PDG data implemented

   Revision 1.10  2002/09/03 14:16:44  allanach
   Taken PRINTOUT, MIXING, EPS out of def.h to make it quicker to compile once
   they are changed.

   Revision 1.9  2002/08/30 12:31:55  allanach
   Test RPV version that works!

   Revision 1.8  2002/07/30 12:57:31  allanach
   SOFTSUSY1.5

   Revision 1.7  2002/07/19 15:54:06  allanach
   SOFTSUSY1.5 version

   Revision 1.6  2002/06/14 16:26:30  allanach
   Switches included for 2-loop running of scalar masses, 
   and calulating mt at mt.

   Revision 1.5  2002/04/18 14:32:05  allanach
   Changed RGEs and anomalous dimensions to be compatible with new notation;
   started implementation of rewsb in R-parity violation

   Revision 1.3  2002/04/12 06:24:50  allanach
   Code maintenance - returning a subobject made simpler

   Revision 1.6  2001/07/18 15:54:50  allanach
   Put MIXING switch into def.h

   Revision 1.5  2001/07/18 14:42:51  allanach
   Added proper header info
   */

#ifndef DEF_H
#define DEF_H

#include <cmath>
const char VERSION[] = "2.0.4";

/// uncomment if you want checking of vector/matrices bounds: slows code down
//#define ARRAY_BOUNDS_CHECKING 

/// Make true if you want to include the 2-loop RGE corrections to scalar mass
/// squared parameters and trilinear terms: they slow it down by a factor of
/// 3. Note that gaugino and Higgs mass parameters are evolved to 2-loops by
/// default anyway.
extern bool INCLUDE_2_LOOP_SCALAR_CORRECTIONS;
/// Set to number of loops to use for calculation of Higgs mass 
/// (currently up to 2, the default)
extern int numHiggsMassLoops;
/// Set to number of loops to use for REWSB condition up to the default of 2
extern int numRewsbLoops;

const double EPSTOL = 1.0e-11; ///< underflow accuracy
const double PI = atan(1.0) * 4.0; ///< or 3.141592653589793 longhand;
const double root2 = sqrt(2.0);

extern double GMU;

/// particle data book 2004 central value. Is just used for intialisation etc
const double MW = 80.410; 
/// particle data book 2004 central value. Is just used for intialisation etc
const double MZ = 91.1876; 
/// variable for level of output and amount of quark: 0-3, higher numbers
/// giving more diagnostics. Set by user in file "massIn"
extern int PRINTOUT;
/// quark mixing flag: set by user in file "massIn":
/// 0=no quark mixing, 1=in up sector, 2=in down sector, -1=3rd family
/// approximation (all at MZ)
extern int MIXING; 
/// overall accuracy required: set by user in file "massIn"
extern double TOLERANCE;
/// SUSY breaking scale - if set by user
extern double QEWSB;

#endif
